@@ -46,7 +46,7 @@ import Germinate.version
 # Where do we get up-to-date seeds from?
 SEEDS = ["http://people.ubuntu.com/~ubuntu-archive/seeds/"]
 SEEDS_BZR = ["http://bazaar.launchpad.net/~ubuntu-core-dev/ubuntu-seeds/"]
-RELEASE = "ubuntu.hardy"
+RELEASE = "ubuntu.intrepid"
 
 # If we need to download Packages.gz and/or Sources.gz, where do we get
 # them from?
@@ -54,7 +54,7 @@ MIRRORS = []
 SOURCE_MIRRORS = []
 DEFAULT_MIRROR = "http://archive.ubuntu.com/ubuntu/"
 DEFAULT_SOURCE_MIRROR = None
-DIST = ["hardy"]
+DIST = ["intrepid"]
 COMPONENTS = ["main", "restricted"]
 ARCH = "i386"
 
@@ -212,7 +212,7 @@ def _write_rdepend_list(f, g, pkg, prefix, stack=None, done=None):
     if "Reverse-Depends" not in g.packages[pkg]:
         return
 
-    for field in ("Pre-Depends", "Depends",
+    for field in ("Pre-Depends", "Depends", "Recommends",
                   "Build-Depends", "Build-Depends-Indep"):
         if field not in g.packages[pkg]["Reverse-Depends"]:
             continue
@@ -257,6 +257,22 @@ def write_seedtext(filename, seedtext):
     for line in seedtext:
         print >>f, line.rstrip('\n')
 
+def write_dot(filename, seednames, seedinherit):
+    """Write a dot file to represent the structure of the seeds"""
+
+    #Initialize dot document
+    dotfile = codecs.open(filename, "w", "utf8", "replace")
+    print >>dotfile, "digraph structure {"
+    print >>dotfile, "    node [color=lightblue2, style=filled];"
+
+    heritage = []
+
+    for seed in seednames:
+        for inherit in seedinherit[seed]:
+            print >>dotfile, "    \"%s\" -> \"%s\";" % (inherit, seed)
+
+    print >>dotfile, "}"
+    dotfile.close()
 
 def usage(f):
     print >>f, """Usage: germinate.py [options]
@@ -391,15 +407,31 @@ def main():
     if os.path.isfile("hints"):
         g.parseHints(open("hints"))
 
-    blacklist = Germinate.seeds.open_seed(SEEDS, RELEASE, "blacklist", bzr)
+    try:
+        blacklist = Germinate.seeds.open_seed(SEEDS, RELEASE, "blacklist", bzr)
+    except Germinate.seeds.SeedError:
+        sys.exit(1)
     if blacklist is not None:
         g.parseBlacklist(blacklist)
 
-    seednames, seedinherit, seedbranches = g.parseStructure(
-        SEEDS, RELEASE, bzr)
+    try:
+        seednames, seedinherit, seedbranches = g.parseStructure(
+            SEEDS, RELEASE, bzr)
+    except Germinate.seeds.SeedError:
+        sys.exit(1)
+
+    write_dot("structure.dot", seednames, seedinherit)
+
+    seednames, seedinherit, seedbranches = g.expandInheritance(
+        seednames, seedinherit, seedbranches)
+
     seedtexts = {}
     for seedname in seednames:
-        seed_fd = Germinate.seeds.open_seed(SEEDS, seedbranches, seedname, bzr)
+        try:
+            seed_fd = Germinate.seeds.open_seed(SEEDS, seedbranches, seedname,
+                                                bzr)
+        except Germinate.seeds.SeedError:
+            sys.exit(1)
         seedtexts[seedname] = seed_fd.readlines()
         seed_fd.close()
         g.plantSeed(seedtexts[seedname],
