@@ -30,6 +30,7 @@ import os
 import getopt
 import logging
 import ConfigParser
+import subprocess
 
 try:
     set # introduced in 2.4
@@ -45,19 +46,28 @@ import Germinate.seeds
 import Germinate.version
 
 def usage(f):
-    print >>f, """Usage: update-metapackage.py [options]
+    print >>f, """Usage: update-metapackage.py [options] [dist]
+
+Update metapackage lists for distribution 'dist' as defined in
+update.cfg.
 
 Options:
 
   -h, --help            Print this help message and exit.
+  -o, --output-directory=DIR
+                        Output in specific directory.
   --version             Output version information and exit.
+  --nodch               Don't modify debian/changelog.
   --bzr                 Fetch seeds using bzr. Requires bzr to be installed.
 """
 
 bzr = False
+nodch = False
+outdir = None
 
 try:
-    opts, args = getopt.getopt(sys.argv[1:], "h", ["help", "version", "bzr"])
+    opts, args = getopt.getopt(sys.argv[1:], "ho:",
+                               ["help", "version", "bzr", "nodch", "output-directory="])
 except getopt.GetoptError:
     usage(sys.stderr)
     sys.exit(2)
@@ -72,6 +82,13 @@ for option, value in opts:
         sys.exit()
     elif option == "--bzr":
         bzr = True
+    elif option == "--nodch":
+        nodch = True
+    elif option in ("-o", "--output-directory"):
+        outdir = value
+
+if not outdir:
+    outdir = "."
 
 if not os.path.exists('debian/control'):
     raise RuntimeError('must be run from the top level of a source package')
@@ -177,15 +194,21 @@ def get_debootstrap_version():
     return version
 
 def debootstrap_packages(arch):
-    debootstrap = os.popen('debootstrap --arch %s --print-debs %s debootstrap-dir %s' % (arch,dist,archive_base[arch][0]))
-    packages = debootstrap.read().split()
-    if debootstrap.close():
-        raise RuntimeError('Unable to retrieve package list from debootstrap')
-    
-    
-    # sometimes debootstrap gives empty packages / multiple separators
-    packages = filter(None, packages)
-    
+    env = dict(os.environ)
+    if 'PATH' in env:
+        env['PATH'] = '/usr/sbin:/sbin:%s' % env['PATH']
+    else:
+        env['PATH'] = '/usr/sbin:/sbin:/usr/bin:/bin'
+    debootstrap = subprocess.Popen(
+        ['debootstrap', '--arch', arch, '--print-debs',
+         dist, 'debootstrap-dir', archive_base[arch][0]],
+        stdout=subprocess.PIPE, env=env, stderr=subprocess.PIPE)
+    (debootstrap_stdout, _) = debootstrap.communicate()
+    if debootstrap.returncode != 0:
+        raise RuntimeError('Unable to retrieve package list from debootstrap: %s' % debootstrap_stdout)    
+
+    packages = filter(None, debootstrap_stdout.split())
+    # sometimes debootstrap gives empty packages / multiple separators   
     packages.sort()
 
     return packages
@@ -205,6 +228,25 @@ def check_debootstrap_version():
 def update_debootstrap_version():
     open(debootstrap_version_file, 'w').write(get_debootstrap_version() + '\n')
 
+def format_changes(items):
+    by_arch = {}
+    for (pkg, arch) in items:
+        by_arch.setdefault(pkg, [])
+        by_arch[pkg].append(arch)
+    all_pkgs = by_arch.keys()
+    all_pkgs.sort()
+    chunks = []
+    for pkg in all_pkgs:
+        arches = by_arch[pkg]
+        if set(architectures) - set(arches):
+            # only some architectures
+            arches.sort()
+            chunks.append('%s [%s]' % (pkg, ' '.join(arches)))
+        else:
+            # all architectures
+            chunks.append(pkg)
+    return ', '.join(chunks)
+
 logger = logging.getLogger()
 logger.setLevel(logging.DEBUG)
 handler = logging.StreamHandler(sys.stdout)
@@ -227,8 +269,13 @@ for architecture in architectures:
     debootstrap_base = set(debootstrap_packages(architecture))
 
     print "[%s] Loading seed lists..." % architecture
-    seed_names, seed_inherit, seed_branches = germinator.parseStructure(
-        seed_base, seed_dist, bzr)
+    try:
+        seed_names, seed_inherit, seed_branches = germinator.parseStructure(
+            seed_base, seed_dist, bzr)
+    except Germinate.seeds.SeedError:
+        sys.exit(1)
+    seed_names, seed_inherit, seed_branches = germinator.expandInheritance(
+        seed_names, seed_inherit, seed_branches)
     needed_seeds = []
     for seed_name in seeds:
         for inherit in seed_inherit[seed_name]:
@@ -238,8 +285,11 @@ for architecture in architectures:
             needed_seeds.append(seed_name)
     seed_texts = {}
     for seed_name in needed_seeds:
-        seed_fd = Germinate.seeds.open_seed(seed_base, seed_branches,
-                                            seed_name, bzr)
+        try:
+            seed_fd = Germinate.seeds.open_seed(seed_base, seed_branches,
+                                                seed_name, bzr)
+        except Germinate.seeds.SeedError:
+            sys.exit(1)
         seed_texts[seed_name] = seed_fd.readlines()
         seed_fd.close()
         germinator.plantSeed(seed_texts[seed_name], architecture, seed_name,
@@ -249,7 +299,7 @@ for architecture in architectures:
     for seed_name in output_seeds:
         meta_name = metapackage_name(seed_name, seed_texts[seed_name])
 
-        output_filename = '%s-%s' % (seed_name,architecture)
+        output_filename = os.path.join(outdir, '%s-%s' % (seed_name,architecture))
         old_list = None
         if os.path.exists(output_filename):
             old_list = set(map(str.strip,open(output_filename).readlines()))
@@ -261,9 +311,10 @@ for architecture in architectures:
                                      seed_texts[seed_name]):
             if package == meta_name:
                 print "%s/%s: Skipping package %s (metapackage)" % (seed_name,architecture,package)
-                continue
-            if seed_name == 'minimal' and package not in debootstrap_base:
+            elif seed_name == 'minimal' and package not in debootstrap_base:
                 print "%s/%s: Skipping package %s (package not in debootstrap)" % (seed_name,architecture,package)
+            elif germinator.packages[package]["Essential"]:
+                print "%s/%s: Skipping package %s (essential)" % (seed_name,architecture,package)
             else:
                 new_list.append(package)
 
@@ -288,7 +339,8 @@ for architecture in architectures:
                 new_recommends_list.append(package)
 
         new_recommends_list.sort()
-        output_recommends_filename = '%s-recommends-%s' % (seed_name,architecture)
+        seed_name_recommends = '%s-recommends' % seed_name
+        output_recommends_filename = os.path.join(outdir, '%s-%s' % (seed_name_recommends,architecture))
         if os.path.exists(output_recommends_filename):
             old_recommends_list = set(map(str.strip,open(output_recommends_filename).readlines()))
             os.rename(output_recommends_filename, output_recommends_filename + '.old')
@@ -325,19 +377,19 @@ for architecture in architectures:
             if value == 1:
                 if recommends_merged.get(package, 0) == -1:
                     moves.setdefault(package,[])
-                    moves[package].append(output_filename)
+                    moves[package].append([seed_name, architecture])
                     recommends_merged[package] += 1
                 else:
                     additions.setdefault(package,[])
-                    additions[package].append(output_filename)
+                    additions[package].append([seed_name, architecture])
             elif value == -1:
                 if recommends_merged.get(package, 0) == 1:
                     moves.setdefault(package,[])
-                    moves[package].append(output_recommends_filename)
+                    moves[package].append([seed_name_recommends, architecture])
                     recommends_merged[package] -= 1
                 else:
                     removals.setdefault(package,[])
-                    removals[package].append(output_filename)
+                    removals[package].append([seed_name, architecture])
 
         mergedrecitems = recommends_merged.items()
         mergedrecitems.sort()
@@ -345,13 +397,12 @@ for architecture in architectures:
             #print package, value
             if value == 1:
                 additions.setdefault(package,[])
-                additions[package].append(output_recommends_filename)
+                additions[package].append([seed_name_recommends, architecture])
             elif value == -1:
                 removals.setdefault(package,[])
-                removals[package].append(output_recommends_filename)
+                removals[package].append([seed_name_recommends, architecture])
 
-
-if additions or removals:
+if not nodch and (additions or removals or moves):
     dch_help = os.popen('dch --help')
     have_U = '-U' in dch_help.read()
     dch_help.close()
@@ -364,12 +415,12 @@ if additions or removals:
     addition_keys.sort()
     for package in addition_keys:
         changes.append('Added %s to %s' %
-                       (package, ', '.join(additions[package])))
+                       (package, format_changes(additions[package])))
     removal_keys = removals.keys()
     removal_keys.sort()
     for package in removal_keys:
         changes.append('Removed %s from %s' %
-                       (package, ', '.join(removals[package])))
+                       (package, format_changes(removals[package])))
     move_keys = moves.keys()
     move_keys.sort()
     for package in move_keys:
@@ -378,10 +429,11 @@ if additions or removals:
         # implemented for depends->recommends or vice versa. In future,
         # using this for moves between seeds might also be useful.
         changes.append('Moved %s to %s' %
-                       (package, ', '.join(moves[package])))
+                       (package, format_changes(moves[package])))
     for change in changes:
         print change
         os.system("dch -a '%s'" % change)
     update_debootstrap_version()
 else:
-    print "No changes found"
+    if not nodch:
+        print "No changes found"
